@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright (c), 2018-2019, SISSA (International School for Advanced Studies).
 # All rights reserved.
@@ -8,9 +7,7 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-from __future__ import unicode_literals
 import locale
-from .compat import PY3
 
 
 class ElementPathError(Exception):
@@ -28,9 +25,6 @@ class ElementPathError(Exception):
         self.token = token
 
     def __str__(self):
-        return unicode(self).encode("utf-8")
-
-    def __unicode__(self):
         if self.code is None:
             return self.message if self.token is None else '%s: %s.' % (self.token, self.message)
         elif self.token is None:
@@ -38,9 +32,6 @@ class ElementPathError(Exception):
         else:
             return '%s: [%s] %s.' % (self.token, self.code, self.message)
 
-    if PY3:
-        __str__ = __unicode__
-
 
 class MissingContextError(ElementPathError):
     """Raised when the dynamic context is required for evaluate the XPath expression."""
