@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright (c), 2018-2019, SISSA (International School for Advanced Studies).
 # All rights reserved.
@@ -35,8 +34,10 @@ XSI_SCHEMA_LOCATION = '{%s}schemaLocation' % XSI_NAMESPACE
 XSI_NONS_SCHEMA_LOCATION = '{%s}schemaLocation' % XSI_NAMESPACE
 
 # XML Schema types
-XSD_NOTATION = '{%s}NOTATION' % XSD_NAMESPACE
+XSD_ANY_TYPE = '{%s}anyType' % XSD_NAMESPACE
+XSD_ANY_SIMPLE_TYPE = '{%s}anySimpleType' % XSD_NAMESPACE
 XSD_ANY_ATOMIC_TYPE = '{%s}anyAtomicType' % XSD_NAMESPACE
+XSD_NOTATION = '{%s}NOTATION' % XSD_NAMESPACE
 XSD_ID = '{%s}ID' % XSD_NAMESPACE
 XSD_IDREF = '{%s}IDREF' % XSD_NAMESPACE
 XSD_IDREFS = '{%s}IDREFS' % XSD_NAMESPACE
