@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright (c), 2018-2019, SISSA (International School for Advanced Studies).
 # All rights reserved.
@@ -8,23 +7,20 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-from __future__ import division
 import math
 import decimal
 
-from .compat import PY3, string_base_type
 from .exceptions import ElementPathSyntaxError, ElementPathNameError, MissingContextError
 from .datatypes import UntypedAtomic, DayTimeDuration, YearMonthDuration, \
     NumericTypeProxy, ArithmeticTypeProxy
 from .xpath_context import XPathSchemaContext
-from .tdop_parser import Parser, MultiLabel
-from .namespaces import XML_ID, XML_LANG, XPATH_FUNCTIONS_NAMESPACE, \
-    XSD_NAMESPACE, XML_NAMESPACE, qname_to_prefixed
+from .tdop_parser import Parser
+from .namespaces import XML_ID, XML_LANG, XML_NAMESPACE, qname_to_prefixed
 from .schema_proxy import AbstractSchemaProxy
 from .xpath_token import XPathToken
-from .xpath_nodes import AttributeNode, NamespaceNode, TypedAttribute, TypedElement,\
-    is_etree_element, is_xpath_node, is_element_node, is_document_node, is_attribute_node, \
-    is_text_node, is_comment_node, is_processing_instruction_node, node_name
+from .xpath_nodes import NamespaceNode, TypedAttribute, TypedElement, is_etree_element, \
+    is_xpath_node, is_element_node, is_document_node, is_attribute_node, is_text_node, \
+    is_comment_node, is_processing_instruction_node, node_name
 
 
 XML_NAME_CHARACTER = (u"A-Z_a-z\u00C0-\u00D6\u00D8-\u00F6\u00F8-\u02FF\u0370-\u037D\u037F-\u1FFF"
@@ -134,11 +130,7 @@ class XPath1Parser(Parser):
             self[:] = self.parser.expression(rbp=bp),
             return self
 
-        axis_pattern_template = r'\b%s(?=\s*\:\:|\s*\(\:.*\:\)\s*\:\:)'
-        try:
-            pattern = axis_pattern_template % symbol.strip()
-        except AttributeError:
-            pattern = axis_pattern_template % getattr(symbol, 'symbol')
+        pattern = r'\b%s(?=\s*\:\:|\s*\(\:.*\:\)\s*\:\:)' % symbol
         return cls.register(symbol, pattern=pattern, label='axis', lbp=bp, rbp=bp, nud=nud_)
 
     @classmethod
@@ -254,9 +246,6 @@ def select(self, context=None):
     name = self.value
     if isinstance(context, XPathSchemaContext):
         # Bind with the XSD type from a schema
-        if name[0] != '{' and self.parser.default_namespace:
-            name = '{%s}%s' % (self.parser.default_namespace, name)
-
         for schema_item in context.iter_children_or_self():
             if self.match_xsd_type(schema_item, name) is not None:
                 yield self.get_typed_node(context, schema_item)
@@ -311,24 +300,8 @@ def led(self, left):
 
     next_token = self.parser.next_token
     if left.symbol == '(name)':
-        try:
-            namespace = self.parser.namespaces[left.value]
-        except KeyError as err:
-            raise self.error('FONS0004', 'No namespace found for prefix %s' % str(err))
-
-        if next_token.symbol not in ('(name)', '*') and next_token.label not in ('function', 'constructor'):
-            next_token.wrong_syntax()
-        elif namespace == XPATH_FUNCTIONS_NAMESPACE:
-            if next_token.label != 'function':
-                next_token.wrong_syntax("An XPath function is expected.")
-            elif isinstance(next_token.label, MultiLabel):
-                next_token.label = 'function'
-        elif namespace == XSD_NAMESPACE:
-            if next_token.symbol not in ('(name)', '*') and next_token.label != 'constructor':
-                next_token.wrong_syntax("An XSD element or a constructor function is expected.")
-            elif isinstance(next_token.label, MultiLabel):
-                next_token.label = 'constructor'
-
+        namespace = self.get_namespace(left.value)
+        next_token.bind_namespace(namespace)
     elif left.symbol == '*' and next_token.symbol != '(name)':
         next_token.wrong_syntax()
 
@@ -340,18 +313,9 @@ def led(self, left):
 
 @method(':')
 def evaluate(self, context=None):
-    if self[0].value == '*':
-        return
-    try:
-        namespace = self.parser.namespaces[self[0].value]
-    except KeyError as err:
-        raise self.error('FONS0004', 'No namespace found for prefix %s' % str(err))
-
-    if namespace == XPATH_FUNCTIONS_NAMESPACE and self[1].label != 'function':
-        self[1].wrong_value("Must be a function")
-    elif namespace == XSD_NAMESPACE and self[1].symbol not in ('(name)', '*') and self[1].label != 'constructor':
-        self[1].wrong_value("An XSD element or a constructor function is expected.")
-    return self[1].evaluate(context)
+    if self[1].label in ('function', 'constructor'):
+        return self[1].evaluate(context)
+    return [x for x in self.select(context)]
 
 
 @method(':')
@@ -359,20 +323,15 @@ def select(self, context=None):
     if self[1].label in ('function', 'constructor'):
         value = self[1].evaluate(context)
         if isinstance(value, list):
-            for result in value:
-                yield result
+            yield from value
         else:
             yield value
         return
     elif self[0].value == '*':
         value = '*:%s' % self[1].value
     else:
-        try:
-            namespace = self.parser.namespaces[self[0].value]
-        except KeyError as err:
-            raise self.error('FONS0004', 'No namespace found for prefix %s' % str(err))
-        else:
-            value = '{%s}%s' % (namespace, self[1].value)
+        namespace = self.get_namespace(self[0].value)
+        value = '{%s}%s' % (namespace, self[1].value)
 
     if context is None:
         return
@@ -412,14 +371,10 @@ def select(self, context=None):
 def nud(self):
     if self.parser.strict:
         self.unexpected()
+
     namespace = self.parser.next_token.value + self.parser.raw_advance('}')
     self.parser.advance()
-
-    next_token = self.parser.next_token
-    if next_token.symbol not in ('(name)', '*') and next_token.label != 'function':
-        next_token.wrong_syntax()
-    elif self.parser.next_token.label != 'function' and namespace == XPATH_FUNCTIONS_NAMESPACE:
-        self.parser.next_token.wrong_syntax()
+    self.parser.next_token.bind_namespace(namespace)
     self[:] = self.parser.symbol_table['(string)'](self.parser, namespace), self.parser.expression(90)
     return self
 
@@ -435,12 +390,7 @@ def evaluate(self, context=None):
 @method('{')
 def select(self, context=None):
     if self[1].label == 'function':
-        value = self[1].evaluate(context)
-        if isinstance(value, list):
-            for result in value:
-                yield result
-        else:
-            yield value
+        yield self[1].evaluate(context)
     elif context is not None:
         value = '{%s}%s' % (self[0].value, self[1].value)
         for item in context.iter_children_or_self():
@@ -456,8 +406,8 @@ def select(self, context=None):
 def nud(self):
     self.parser.next_token.expected('(name)')
     self[:] = self.parser.expression(rbp=90),
-    if self[0].value.startswith('{'):
-        self[0].wrong_value("Variable reference requires a simple reference name")
+    if self[0].value.startswith(':'):
+        self[0].wrong_syntax("Variable reference requires a simple reference name")
     return self
 
 
@@ -570,20 +520,15 @@ def evaluate(self, context=None):
 ###
 # Numerical operators
 prefix('+')
-prefix('-', bp=90)
+prefix('-', bp=70)
 
 
 @method(infix('+', bp=40))
 def evaluate(self, context=None):
-    if not self:
-        return
-    elif len(self) == 1:
+    if len(self) == 1:
         arg = self.get_argument(context, cls=NumericTypeProxy)
         if arg is not None:
-            try:
-                return +arg
-            except TypeError:
-                raise self.wrong_type("numeric value is required: %r" % arg)
+            return +arg
     else:
         op1, op2 = self.get_operands(context, cls=ArithmeticTypeProxy)
         if op1 is not None:
@@ -598,10 +543,7 @@ def evaluate(self, context=None):
     if len(self) == 1:
         arg = self.get_argument(context, cls=NumericTypeProxy)
         if arg is not None:
-            try:
-                return -arg
-            except TypeError:
-                raise self.wrong_type("numeric value is required: %r" % arg)
+            return -arg
     else:
         op1, op2 = self.get_operands(context, cls=ArithmeticTypeProxy)
         if op1 is not None:
@@ -667,12 +609,10 @@ def select(self, context=None):
         return
     elif not self.cut_and_sort:
         for k in range(2):
-            for item in self[k].select(context.copy()):
-                yield item
+            yield from self[k].select(context.copy())
     else:
         results = {item for k in range(2) for item in self[k].select(context.copy())}
-        for item in context.iter_results(results):
-            yield item
+        yield from context.iter_results(results)
 
 
 ###
@@ -717,8 +657,7 @@ def select(self, context=None):
             yield context.root
     elif len(self) == 1:
         context.item = None
-        for result in self[0].select(context):
-            yield result
+        yield from self[0].select(context)
     else:
         items = []
         left_results = [x for x in self[0].select(context)]
@@ -735,9 +674,6 @@ def select(self, context=None):
                     if result[0] not in items:
                         items.append(result)
                         yield result
-                elif isinstance(result, AttributeNode):
-                    items.append(result)
-                    yield result
                 else:
                     items.append(result)
                     yield result
@@ -750,15 +686,13 @@ def select(self, context=None):
     elif len(self) == 1:
         context.item = None
         for _ in context.iter_descendants(axis='descendant-or-self'):
-            for result in self[0].select(context):
-                yield result
+            yield from self[0].select(context)
     else:
         for elem in self[0].select(context):
             if not is_element_node(elem):
                 self.wrong_type("left operand must returns element nodes: %r" % elem)
             for _ in context.iter_descendants(item=elem):
-                for result in self[1].select(context):
-                    yield result
+                yield from self[1].select(context)
 
 
 ###
@@ -809,16 +743,14 @@ def select(self, context=None):
 def select(self, context=None):
     if context is not None:
         for _ in context.iter_self():
-            for result in self[0].select(context):
-                yield result
+            yield from self[0].select(context)
 
 
 @method(axis('child'))
 def select(self, context=None):
     if context is not None:
         for _ in context.iter_children_or_self(child_axis=True):
-            for result in self[0].select(context):
-                yield result
+            yield from self[0].select(context)
 
 
 @method(axis('descendant'))
@@ -827,16 +759,14 @@ def select(self, context=None):
         item = context.item
         for _ in context.iter_descendants(axis=self.symbol):
             if item is not context.item:
-                for result in self[0].select(context):
-                    yield result
+                yield from self[0].select(context)
 
 
 @method(axis('descendant-or-self'))
 def select(self, context=None):
     if context is not None:
         for _ in context.iter_descendants(axis=self.symbol):
-            for result in self[0].select(context):
-                yield result
+            yield from self[0].select(context)
 
 
 @method(axis('following-sibling'))
@@ -852,15 +782,13 @@ def select(self, context=None):
         for parent in context.iter_parent(axis=self.symbol):
             if isinstance(context, XPathSchemaContext):
                 for _ in context.iter_children_or_self(parent, child_axis=True):
-                    for result in self[0].select(context):
-                        yield result
+                    yield from self[0].select(context)
 
             else:
                 follows = False
                 for child in context.iter_children_or_self(parent, child_axis=True):
                     if follows:
-                        for result in self[0].select(context):
-                            yield result
+                        yield from self[0].select(context)
                     elif item is child:
                         follows = True
 
@@ -874,8 +802,7 @@ def select(self, context=None):
         for elem in context.iter_descendants(item=context.root, axis=self.symbol):
             if follows:
                 if elem not in descendants:
-                    for result in self[0].select(context):
-                        yield result
+                    yield from self[0].select(context)
             elif item is elem:
                 follows = True
 
@@ -895,8 +822,7 @@ def select(self, context=None):
         self.missing_context()
 
     for _ in context.iter_attributes():
-        for result in self[0].select(context):
-            yield result
+        yield from self[0].select(context)
 
 
 @method(axis('namespace'))
@@ -910,7 +836,7 @@ def select(self, context=None):
             yield context.item
 
         if hasattr(elem, 'nsmap'):
-            # Maybe an lxml's Element: don't use parser namespaces for axis.
+            # Add element's namespaces for lxml (and use None for default namespace)
             for prefix_, uri in elem.nsmap.items():
                 if prefix_ not in namespaces:
                     context.item = NamespaceNode(prefix_, uri)
@@ -923,8 +849,7 @@ def select(self, context=None):
 def select(self, context=None):
     if context is not None:
         for _ in context.iter_parent(axis=self.symbol):
-            for result in self[0].select(context):
-                yield result
+            yield from self[0].select(context)
 
 
 @method(axis('ancestor'))
@@ -960,16 +885,14 @@ def select(self, context=None):
                     break
                 else:
                     context.item = child
-                    for result in self[0].select(context):
-                        yield result
+                    yield from self[0].select(context)
 
 
 @method(axis('preceding'))
 def select(self, context=None):
     if context is not None and is_element_node(context.item):
         for _ in context.iter_preceding():
-            for result in self[0].select(context):
-                yield result
+            yield from self[0].select(context)
 
 
 ###
@@ -1027,9 +950,7 @@ def select(self, context=None):
         value = self[0].evaluate(context)
         item = context.item
         if is_element_node(item):
-            for elem in item.iter():
-                if elem.get(XML_ID) == value:
-                    yield elem
+            yield from filter(lambda e: e.get(XML_ID) == value, item.iter())
 
 
 @method(function('name', nargs=(0, 1)))
@@ -1060,8 +981,8 @@ def evaluate(self, context=None):
 
 @method(function('contains', nargs=2))
 def evaluate(self, context=None):
-    arg1 = self.get_argument(context, default='', cls=string_base_type)
-    arg2 = self.get_argument(context, index=1, default='', cls=string_base_type)
+    arg1 = self.get_argument(context, default='', cls=str)
+    arg2 = self.get_argument(context, index=1, default='', cls=str)
     return arg2 in arg1
 
 
@@ -1073,7 +994,7 @@ def evaluate(self, context=None):
 
 @method(function('string-length', nargs=(0, 1)))
 def evaluate(self, context=None):
-    return len(self.get_argument(context, default_to_context=True, default='', cls=string_base_type))
+    return len(self.get_argument(context, default_to_context=True, default='', cls=str))
 
 
 @method(function('normalize-space', nargs=(0, 1)))
@@ -1081,48 +1002,37 @@ def evaluate(self, context=None):
     if self.parser.version == '1.0':
         arg = self.string_value(self.get_argument(context, default_to_context=True, default=''))
     else:
-        arg = self.get_argument(context, default_to_context=True, default='', cls=string_base_type)
+        arg = self.get_argument(context, default_to_context=True, default='', cls=str)
     return ' '.join(arg.strip().split())
 
 
 @method(function('starts-with', nargs=2))
 def evaluate(self, context=None):
-    arg1 = self.get_argument(context, default='', cls=string_base_type)
-    arg2 = self.get_argument(context, index=1, default='', cls=string_base_type)
+    arg1 = self.get_argument(context, default='', cls=str)
+    arg2 = self.get_argument(context, index=1, default='', cls=str)
     return arg1.startswith(arg2)
 
 
 @method(function('translate', nargs=3))
 def evaluate(self, context=None):
-    arg = self.get_argument(context, default='', cls=string_base_type)
-    map_string = self.get_argument(context, index=1, default='', cls=string_base_type)
-    trans_string = self.get_argument(context, index=2, default='', cls=string_base_type)
+    arg = self.get_argument(context, default='', cls=str)
+    map_string = self.get_argument(context, index=1, default='', cls=str)
+    trans_string = self.get_argument(context, index=2, default='', cls=str)
 
-    if not PY3:
-        import string
-        maketrans = getattr(string, 'maketrans')
-        arg = arg.encode('utf-8')
-        map_string = map_string.encode('utf-8')
-        trans_string = trans_string.encode('utf-8')
-    else:
-        maketrans = str.maketrans
+    maketrans = str.maketrans
 
     if len(map_string) == len(trans_string):
         return arg.translate(maketrans(map_string, trans_string))
     elif len(map_string) > len(trans_string):
         k = len(trans_string)
-        if PY3:
-            return arg.translate(maketrans(map_string[:k], trans_string, map_string[k:]))
-        for c in map_string[k:]:
-            arg = arg.replace(c, '')
-        return arg.translate(maketrans(map_string[:k], trans_string))
+        return arg.translate(maketrans(map_string[:k], trans_string, map_string[k:]))
     else:
         self.wrong_value("the third argument must have a length less or equal than the second")
 
 
 @method(function('substring', nargs=(2, 3)))
 def evaluate(self, context=None):
-    item = self.get_argument(context, default='', cls=string_base_type)
+    item = self.get_argument(context, default='', cls=str)
     start = self.get_argument(context, index=1)
     try:
         if math.isnan(start) or math.isinf(start):
@@ -1154,8 +1064,8 @@ def evaluate(self, context=None):
 @method(function('substring-before', nargs=2))
 @method(function('substring-after', nargs=2))
 def evaluate(self, context=None):
-    arg1 = self.get_argument(context, default='', cls=string_base_type)
-    arg2 = self.get_argument(context, index=1, default='', cls=string_base_type)
+    arg1 = self.get_argument(context, default='', cls=str)
+    arg2 = self.get_argument(context, index=1, default='', cls=str)
     if arg1 is None:
         return ''
 
@@ -1283,10 +1193,8 @@ def evaluate(self, context=None):
         number = decimal.Decimal(arg)
         if number > 0:
             return float(number.quantize(decimal.Decimal('1'), rounding='ROUND_HALF_UP'))
-        elif PY3:
-            return float(round(number))
         else:
-            return float(number.quantize(decimal.Decimal('1'), rounding='ROUND_HALF_DOWN'))
+            return float(round(number))
     except TypeError as err:
         self.wrong_type(str(err))
     except decimal.DecimalException as err:
