@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright (c), 2018-2019, SISSA (International School for Advanced Studies).
 # All rights reserved.
@@ -9,11 +8,11 @@
 # @author Davide Brunato <brunato@sissa.it>
 #
 import datetime
+from functools import lru_cache
 
-from .compat import lru_cache
 from .exceptions import ElementPathTypeError
-from .xpath_nodes import AttributeNode, TypedAttribute, TypedElement, is_etree_element, \
-    is_element_node, is_document_node, is_attribute_node
+from .xpath_nodes import AttributeNode, TextNode, TypedAttribute, TypedElement, \
+    is_etree_element, is_element_node, is_document_node, is_attribute_node
 
 
 class XPathContext(object):
@@ -180,7 +179,7 @@ class XPathContext(object):
         elif is_etree_element(self.item):
             elem = self.item
             if elem.text is not None:
-                self.item = elem.text
+                self.item = TextNode(elem.text)
                 yield self.item
             self.size = len(elem)
             for self.position, self.item in enumerate(elem):
@@ -246,8 +245,7 @@ class XPathContext(object):
         elif not is_element_node(self.item):
             return
 
-        for descendant in self._iter_descendants():
-            yield descendant
+        yield from self._iter_descendants()
 
         self.item, self.size, self.position, self.axis = status
 
@@ -255,13 +253,12 @@ class XPathContext(object):
         elem = self._elem = self.item
         yield elem
         if elem.text is not None:
-            self.item = elem.text
+            self.item = TextNode(elem.text)
             yield self.item
         if len(elem):
             self.size = len(elem)
             for self.position, self.item in enumerate(elem):
-                for item in self._iter_descendants():
-                    yield item
+                yield from self._iter_descendants()
 
     def iter_ancestors(self, item=None, axis=None):
         status = self.item, self.size, self.position, self.axis
@@ -290,8 +287,7 @@ class XPathContext(object):
             yield self.item
             self.item = self.item.getroot()
 
-        for item in self._iter_context():
-            yield item
+        yield from self._iter_context()
 
         self.item, self.size, self.position, = status
 
@@ -320,7 +316,7 @@ class XPathContext(object):
         elem = self._elem = self.item
         yield elem
         if elem.text is not None:
-            self.item = elem.text
+            self.item = TextNode(elem.text)
             yield self.item
 
         for self.item in map(lambda x: AttributeNode(*x), elem.attrib.items()):
@@ -329,8 +325,7 @@ class XPathContext(object):
         if len(elem):
             self.size = len(elem)
             for self.position, self.item in enumerate(elem):
-                for item in self._iter_context():
-                    yield item
+                yield from self._iter_context()
 
 
 class XPathSchemaContext(XPathContext):
