@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Copyright (c), 2018-2019, SISSA (International School for Advanced Studies).
 # All rights reserved.
@@ -19,26 +18,28 @@ In XPath there are 7 kinds of nodes:
 Element-like objects are used for representing elements and comments, ElementTree-like objects
 for documents. Generic tuples are used for representing attributes and named-tuples for namespaces.
 """
-from __future__ import unicode_literals
 import locale
 import contextlib
 from decimal import Decimal
+import urllib.parse
 
-from .compat import string_base_type, unicode_type, urljoin, urlparse, uses_relative
 from .exceptions import xpath_error
-from .namespaces import XQT_ERRORS_NAMESPACE
-from .xpath_nodes import AttributeNode, TypedAttribute, TypedElement, \
+from .namespaces import XQT_ERRORS_NAMESPACE, XSD_NAMESPACE, XPATH_FUNCTIONS_NAMESPACE, \
+    XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE
+from .xpath_nodes import AttributeNode, TextNode, TypedAttribute, TypedElement, \
     is_etree_element, is_attribute_node, elem_iter_strings, is_text_node, \
     is_namespace_node, is_comment_node, is_processing_instruction_node, \
     is_element_node, is_document_node, is_xpath_node, is_schema_node
 from .datatypes import UntypedAtomic, Timezone, DayTimeDuration, XSD_BUILTIN_TYPES
 from .schema_proxy import AbstractSchemaProxy
-from .tdop_parser import Token
+from .tdop_parser import Token, MultiLabel
 from .xpath_context import XPathSchemaContext
 
 
 UNICODE_CODEPOINT_COLLATION = "http://www.w3.org/2005/xpath-functions/collation/codepoint"
 
+XSD_SPECIAL_TYPES = {XSD_ANY_TYPE, XSD_ANY_SIMPLE_TYPE, XSD_ANY_ATOMIC_TYPE}
+
 
 def ordinal(n):
     if n in {11, 12, 13}:
@@ -77,8 +78,7 @@ class XPathToken(Token):
         item = self.evaluate(context)
         if item is not None:
             if isinstance(item, list):
-                for _item in item:
-                    yield _item
+                yield from item
             else:
                 if context is not None:
                     context.item = item
@@ -158,6 +158,9 @@ class XPathToken(Token):
                 return
         else:
             item = None
+            if context is not None:
+                context = context.copy()
+
             for k, result in enumerate(selector(context)):
                 if k == 0:
                     item = result
@@ -175,7 +178,7 @@ class XPathToken(Token):
         # Type promotion checking (see "function conversion rules" in XPath 2.0 language definition)
         if cls is not None and not isinstance(item, cls):
             if self.parser.compatibility_mode:
-                if issubclass(cls, string_base_type):
+                if issubclass(cls, str):
                     return self.string_value(item)
                 elif issubclass(cls, float) or issubclass(float, cls):
                     return self.number_value(item)
@@ -186,7 +189,7 @@ class XPathToken(Token):
                     return value
                 elif isinstance(value, UntypedAtomic):
                     try:
-                        if issubclass(cls, string_base_type):
+                        if issubclass(cls, str):
                             return str(value)
                         else:
                             return cls(value)
@@ -234,12 +237,15 @@ class XPathToken(Token):
                     value = str(value)
                 if isinstance(context, XPathSchemaContext):
                     return value
-                if self.xsd_type is not None and isinstance(value, string_base_type):
-                    try:
-                        value = self.xsd_type.decode(value)
-                    except (TypeError, ValueError):
-                        msg = "Type {!r} is not appropriate for the context"
-                        self.wrong_context_type(msg.format(type(value)))
+                if self.xsd_type is not None and isinstance(value, str):
+                    if self.xsd_type.name in XSD_SPECIAL_TYPES:
+                        value = UntypedAtomic(value)
+                    else:
+                        try:
+                            value = self.xsd_type.decode(value)
+                        except (TypeError, ValueError):
+                            msg = "Type {!r} is not appropriate for the context"
+                            self.wrong_context_type(msg.format(type(value)))
                 return value
             else:
                 self.wrong_context_type("atomized operand is a sequence of length greater than one")
@@ -290,7 +296,7 @@ class XPathToken(Token):
         :param context: the XPath dynamic context.
         """
         for result in self.select(context):
-            if isinstance(result, TypedElement):
+            if isinstance(result, (TypedElement, TextNode)):
                 yield result[0]
             elif isinstance(result, AttributeNode):
                 yield result[1]
@@ -359,11 +365,41 @@ class XPathToken(Token):
         obtained by the join o the base_uri of the static context with the
         argument. Returns the argument if the base_uri is `None'.
         """
-        url_parts = urlparse(uri)
-        if url_parts.scheme not in uses_relative or url_parts.path.startswith('/') \
+        url_parts = urllib.parse.urlparse(uri)
+        if url_parts.scheme not in urllib.parse.uses_relative \
+                or url_parts.path.startswith('/') \
                 or self.parser.base_uri is None:
             return uri
-        return urljoin(self.parser.base_uri, uri)
+        return urllib.parse.urljoin(self.parser.base_uri, uri)
+
+    def get_namespace(self, prefix):
+        """
+        Resolves a prefix to a namespace raising an error (FONS0004) if the
+        prefix is not found in the namespace map.
+        """
+        try:
+            return self.parser.namespaces[prefix]
+        except KeyError as err:
+            raise self.error('FONS0004', 'No namespace found for prefix %s' % str(err))
+
+    def bind_namespace(self, namespace):
+        """
+        Bind a token with a namespace. The token has to be a name, a name wildcard,
+        a function or a constructor, otherwise a syntax error is raised. Functions
+        and constructors must be limited to its namespaces.
+        """
+        if self.symbol not in ('(name)', '*') and self.label not in ('function', 'constructor'):
+            self.wrong_syntax()
+        elif namespace == XPATH_FUNCTIONS_NAMESPACE:
+            if self.label != 'function':
+                self.wrong_syntax("An XPath function is expected.")
+            elif isinstance(self.label, MultiLabel):
+                self.label = 'function'
+        elif namespace == XSD_NAMESPACE:
+            if self.symbol not in ('(name)', '*') and self.label != 'constructor':
+                self.wrong_syntax("An XSD element or a constructor function is expected.")
+            elif isinstance(self.label, MultiLabel):
+                self.label = 'constructor'
 
     def adjust_datetime(self, context, cls):
         """
@@ -464,9 +500,16 @@ class XPathToken(Token):
         else:
             try:
                 if isinstance(item, AttributeNode):
-                    return TypedAttribute(item, self.xsd_type.decode(item[1]))
+                    if self.xsd_type.name in XSD_SPECIAL_TYPES:
+                        return TypedAttribute(item, UntypedAtomic(item[1]))
+                    else:
+                        return TypedAttribute(item, self.xsd_type.decode(item[1]))
+
                 elif self.xsd_type.is_simple() or self.xsd_type.has_simple_content():
-                    return TypedElement(item, self.xsd_type.decode(item.text))
+                    if self.xsd_type.name in XSD_SPECIAL_TYPES:
+                        return TypedElement(item, UntypedAtomic(item.text))
+                    else:
+                        return TypedElement(item, self.xsd_type.decode(item.text))
                 else:
                     return item
             except (TypeError, ValueError):
@@ -537,9 +580,9 @@ class XPathToken(Token):
         elif is_element_node(obj):
             return ''.join(elem_iter_strings(obj))
         elif is_attribute_node(obj):
-            return unicode_type(obj[1])
+            return str(obj[1])
         elif is_text_node(obj):
-            return obj
+            return obj[0]
         elif is_document_node(obj):
             return ''.join(e.text for e in obj.getroot().iter() if e.text is not None)
         elif is_namespace_node(obj):
